require(rgdal)

#
# Separate the get data functions from the processing functions
# to more easily set debugging data not available flags
#
get_DEM=function(clon, clat, wsext,product="us10m"){
require(XML)
require(raster)

bbox=paste0(clon-wsext,",",clat-wsext,",",clon+wsext,",",clat+wsext)
productsource="UUID-3e908b88-8788-4676-bc1e-e47b46e38763" # 10m DEM


demdirurl=paste0("http://gradlab4.bse.vt.edu:8080/gi-cat-10.0.2//services/opensearchsemanticenhanced?&reqID=npzmhOwDsB4bbF3UZ9EUPQ1R9b3ViTIV&radio_group=on&bbox=",bbox,"&relation_radio_group=on&rel=overlaps&ct=1000&outputFormat=application%2Fatom%2Bxml&evtOrd=Time&sources=",productsource)

dems.str<-htmlTreeParse(demdirurl,useInternalNodes=T)
demurls=grep("wcs?.*OPeNDAP",xpathSApply(dems.str,"//url",xmlValue),value=T)
print(length(demurls))

wcsendstr=paste0("&SERVICE=WCS&SERVICE=WCS&VERSION=1.0.0&REQUEST=GetCoverage&COVERAGE=Band1&FORMAT=GeoTIFF&BBOX=",bbox,"&CRS=EPSG:4326")
# *Note Areas which were on a lat/lon boundary that require multiple tiles are merged using R merge command ***?merge
demlist=list()
for(i in 1:length(demurls)){
 demurl=paste0(demurls[i],wcsendstr)
 print(demurl)
 download.file(demurl,paste0(i,"dem.tif"),mode="wb")
 demtmp=raster(paste0(i,"dem.tif"))
 demlist[[i]]=demtmp
}
if(length(demlist)==1){
fulldem <- demlist[[1]]
} else {
fulldem <- do.call(merge, demlist)
}

writeRaster(fulldem,"fulldem.tif","GTiff",options=c("COMPRESS=NONE", "TFW=YES"),overwrite=T, datatype='FLT4S')

fulldemUTM <- projectRaster(fulldem, crs=utmproj4)
values(fulldemUTM)[is.na(values(fulldemUTM))]=min(values(fulldemUTM),na.rm=T)
writeRaster(fulldemUTM,"fulldemUTM.tif","GTiff",options=c("COMPRESS=NONE", "TFW=YES"),overwrite=T, datatype='FLT4S')

return(fulldemUTM)
}

#
# This function is needed to swap lat/lon axis due to a swapped
#   axis order issue between WFS 1.0.0 and 1.1.0+ servers
#
swapAxisSpDf<- function(sp.df)
{
   for(j in 1:length(sp.df@polygons)){
    for(i in 1:length(sp.df@polygons[[1]]@Polygons))
    {
        y=sp.df@polygons[[j]]@Polygons[[i]]@coords[,1]
        x=sp.df@polygons[[j]]@Polygons[[i]]@coords[,2]
        sp.df@polygons[[j]]@Polygons[[i]]@coords[,1]=x
        sp.df@polygons[[j]]@Polygons[[i]]@coords[,2]=y
        x=sp.df@polygons[[j]]@Polygons[[i]]@labpt[1]
        sp.df@polygons[[j]]@Polygons[[i]]@labpt[1]=sp.df@polygons[[j]]@Polygons[[i]]@labpt[2]
        sp.df@polygons[[j]]@Polygons[[i]]@labpt[2]=x
    }
    x=sp.df@polygons[[j]]@labpt[1]
    sp.df@polygons[[j]]@labpt[1]=sp.df@polygons[[j]]@labpt[2]
    sp.df@polygons[[j]]@labpt[2]=x
   }
    x=sp.df@bbox[1,]
    sp.df@bbox[1,]=sp.df@bbox[2,]
    sp.df@bbox[2,]=x
    return(sp.df)
mapunitpoly2=swapAxisSpDf(mapunitpoly)
writeOGR(mapunitpoly2, ".", "mapunitpoly", driver="ESRI Shapefile",overwrite=T)
return(mapunitpoly2)

}

readmdb=function(mdbname=NULL,tablename=NULL){

  print(Sys.info()["sysname"])
  if(Sys.info()["sysname"]=="Windows"){require("RODBC")} else {require(Hmic)}

}
#
# Need to add in more testing to confirm things are installed here. 
#
setup_TauRk=function(){
  reqdPackages=c("rgeos","EcoHydRology","RQGIS","raster","XML","rgdal","classInt")
  for(reqdlib in reqdPackages){
   if (!require(reqdlib,character.only=T)) { 
     print(reqdlib)
     install.packages(reqdlib, dependencies = TRUE); 
     require(reqdlib,character.only=T) 
   }
  }

 TauRk_env<- set_env()
 if(Sys.info()["sysname"]=="Windows"){
  mpiexec_cmd=system("where mpiexec",intern=T)
  subpath="/.."
  cmdend=".exe$"
 }else{
  mpiexec_cmd=list.files(recursive=T,path=TauRk_env$qgis_prefix_path,pattern="mpiexec",full.names=T)[1]
  subpath=""
  cmdend="$"
 }
 
 TauDEMcmdNames=c( "pitremove", "d8flowdir", "aread8", "threshold", "moveoutletstostrm", "dinfflowdir", "areadinf", "streamnet")

 for (TDCN in TauDEMcmdNames){
   TDCNcmd=TDCN
   print(paste0(TauRk_env$root,subpath))
   print(paste0(TDCN,cmdend))
   if(Sys.info()["sysname"]=="Windows" & TDCNcmd=="moveoutletstostrm"){TDCNcmd="moveoutletstostreams"}

   tmpcmd=list.files(recursive=T,path=paste0(TauRk_env$root,subpath),pattern=paste0(TDCNcmd,cmdend),full.names=T,ignore.case=T)[1]
   TauRk_env[paste0(TDCN,"_cmd")]=tmpcmd
   print(tmpcmd)

 } 
 my_env$mpiexec_cmd=mpiexec_cmd
 print(mpiexec_cmd)
 print(class(mpiexec_cmd))
 return(my_env)
}

CreateOutlet=function(outclon=clon, outclat=clat){
 outletptLL=data.frame(outlet="outlet",clat=outclat,clon=outclon)
 coordinates(outletptLL) <- c("clon", "clat")
 proj4string(outletptLL)=CRS("+proj=longlat +datum=WGS84")
 utmproj4 = paste0("+proj=utm +zone=", trunc((180+clon)/6+1), " +datum=WGS84 +units=m +no_defs")
 outletptUTM <- spTransform(outletptLL, CRS(utmproj4))
 writeOGR(outletptUTM, ".", "outlet", driver="ESRI Shapefile",overwrite=T)
 return(outletptUTM)
}

ProcessGrids=function(){
#
# Run setup_TauRk() first
#
unlink(c("fel.tif"))
### Error related to TauDEM writing bad file first time... fix is to create an empty file larger
### than the tiff will be. dd count=larger than dem, if=/dev/zero of=fel.tif
felsyscmd=paste0(shQuote(TauRk_env$mpiexec_cmd)," -n 2 ",shQuote(TauRk_env$pitremove_cmd)," -z fulldemUTM.tif -fel fel.tif")
system(felsyscmd); system(felsyscmd)
plot(raster("fel.tif"),colNA="black")

unlink(c("p.tif","sd8.tif"))
sd8syscmd=paste0(shQuote(TauRk_env$mpiexec_cmd)," -n 2 ",shQuote(TauRk_env$d8flowdir_cmd)," -fel fel.tif -p p.tif -sd8 sd8.tif")
system(sd8syscmd,show.output.on.console=T) :system(sd8syscmd,show.output.on.console=T)
print(sd8syscmd)
plot(raster("p.tif"),colNA="black")
plot(raster("sd8.tif"),colNA="black")

unlink(c("ad8.tif"))
ad8syscmd=paste0(shQuote(TauRk_env$mpiexec_cmd)," -n 2 ",shQuote(TauRk_env$aread8_cmd)," -p p.tif -ad8 ad8.tif")
print(ad8syscmd)
system(ad8syscmd);system(ad8syscmd)
plot(raster("ad8.tif"),colNA="black")

thresh=floor(cellStats(raster("ad8.tif"),max)*.05)
thressyscmd=paste0(shQuote(TauRk_env$mpiexec_cmd)," -n 2 ",shQuote(TauRk_env$threshold_cmd)," -ssa ad8.tif -thresh ",thresh," -src src.tif")
print(thressyscmd)
system(thressyscmd);system(thressyscmd,show.output.on.console=T)
plot(raster("ad8.tif"),colNA="black")

#
# TI Calculation
#
dinfflowdircmd=paste0(shQuote(TauRk_env$mpiexec_cmd)," -n 2 ",shQuote(TauRk_env$dinfflowdir_cmd)," -fel fel.tif -ang ang.tif -slp slp.tif")
print(dinfflowdircmd)
system(dinfflowdircmd);system(dinfflowdircmd,show.output.on.console=T)

areadinfcmd=paste0(shQuote(TauRk_env$mpiexec_cmd)," -n 2 ",shQuote(TauRk_env$areadinf_cmd)," -ang ang.tif -sca sca.tif")
print(areadinfcmd)
system(areadinfcmd);system(areadinfcmd,show.output.on.console=T)
sca=raster("sca.tif")
slp=raster("slp.tif")
minslp=min(values(slp)[values(slp)>0],na.rm=T)
values(slp)=values(slp)+minslp
TI=log((sca+1)/slp)
writeRaster(TI,"TI.tif","GTiff",options=c("COMPRESS=NONE", "TFW=YES"),overwrite=T, datatype='FLT4S')

nTIclass=10
v=values(TI)
v=v[!is.na(v)]
brks.qt = classIntervals(v, n = nTIclass, style = "quantile")$brks #length nTIclass+1 of just the numeric breakpoints
TIC = cut(TI, breaks=brks.qt, include.lowest = T, right=T)
writeRaster(TIC,"TIC.tif","GTiff",options=c("COMPRESS=NONE", "TFW=YES"),overwrite=T, datatype='FLT4S')
plot(TIC,colNA="black")
TIC_sp <- as(TIC, 'SpatialGridDataFrame')

return(over(outletptUTM,TIC_sp))
}

